#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<IcingaDB>::TypeImpl()
{ }

TypeImpl<IcingaDB>::~TypeImpl()
{ }

String TypeImpl<IcingaDB>::GetName() const
{
	return "IcingaDB";
}

int TypeImpl<IcingaDB>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<IcingaDB>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<IcingaDB>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 100:
			if (name == "db_index")
				return offset + 4;

			break;
		case 104:
			if (name == "host")
				return offset + 0;

			break;
		case 112:
			if (name == "path")
				return offset + 1;
			if (name == "password")
				return offset + 2;
			if (name == "port")
				return offset + 3;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<IcingaDB>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "host", "host", nullptr, 2, 0};
		case 1:
			return {1, "String", "path", "path", nullptr, 2, 0};
		case 2:
			return {2, "String", "password", "password", nullptr, 2, 0};
		case 3:
			return {3, "Number", "port", "port", nullptr, 2, 0};
		case 4:
			return {4, "Number", "db_index", "db_index", nullptr, 2, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<IcingaDB>::GetFieldCount() const
{
	return 5 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<IcingaDB>::GetFactory() const
{
	return TypeHelper<IcingaDB, false>::GetFactory();
}

std::vector<String> TypeImpl<IcingaDB>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<IcingaDB>::GetActivationPriority() const
{
	return 0;
}

void TypeImpl<IcingaDB>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<IcingaDB>::OnHostChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<IcingaDB>::OnPathChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<IcingaDB>::OnPasswordChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<IcingaDB>::OnPortChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<IcingaDB>::OnDbIndexChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<IcingaDB>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateHost(Lazy<String>([this]() { return GetHost(); }), utils);
	if (2 & types)
		ValidatePath(Lazy<String>([this]() { return GetPath(); }), utils);
	if (2 & types)
		ValidatePassword(Lazy<String>([this]() { return GetPassword(); }), utils);
	if (2 & types)
		ValidatePort(Lazy<int>([this]() { return GetPort(); }), utils);
	if (2 & types)
		ValidateDbIndex(Lazy<int>([this]() { return GetDbIndex(); }), utils);
}

void ObjectImpl<IcingaDB>::SimpleValidateHost(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<IcingaDB>::SimpleValidatePath(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<IcingaDB>::SimpleValidatePassword(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<IcingaDB>::SimpleValidatePort(const Lazy<int>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<IcingaDB>::SimpleValidateDbIndex(const Lazy<int>& value, const ValidationUtils& utils)
{
}

ObjectImpl<IcingaDB>::ObjectImpl()
{
	SetHost(GetDefaultHost(), true);
	SetPath(GetDefaultPath(), true);
	SetPassword(GetDefaultPassword(), true);
	SetPort(GetDefaultPort(), true);
	SetDbIndex(GetDefaultDbIndex(), true);
}

ObjectImpl<IcingaDB>::~ObjectImpl()
{ }

void ObjectImpl<IcingaDB>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetHost(value, suppress_events, cookie);
			break;
		case 1:
			SetPath(value, suppress_events, cookie);
			break;
		case 2:
			SetPassword(value, suppress_events, cookie);
			break;
		case 3:
			SetPort(value, suppress_events, cookie);
			break;
		case 4:
			SetDbIndex(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<IcingaDB>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetHost();
		case 1:
			return GetPath();
		case 2:
			return GetPassword();
		case 3:
			return GetPort();
		case 4:
			return GetDbIndex();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<IcingaDB>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateHost(lvalue, utils);
			break;
		case 1:
			ValidatePath(lvalue, utils);
			break;
		case 2:
			ValidatePassword(lvalue, utils);
			break;
		case 3:
			ValidatePort(lvalue, utils);
			break;
		case 4:
			ValidateDbIndex(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<IcingaDB>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyHost(cookie);
			break;
		case 1:
			NotifyPath(cookie);
			break;
		case 2:
			NotifyPassword(cookie);
			break;
		case 3:
			NotifyPort(cookie);
			break;
		case 4:
			NotifyDbIndex(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<IcingaDB>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<IcingaDB>::GetHost() const
{
	return m_Host;
}

String ObjectImpl<IcingaDB>::GetPath() const
{
	return m_Path;
}

String ObjectImpl<IcingaDB>::GetPassword() const
{
	return m_Password;
}

int ObjectImpl<IcingaDB>::GetPort() const
{
	return m_Port;
}

int ObjectImpl<IcingaDB>::GetDbIndex() const
{
	return m_DbIndex;
}

void ObjectImpl<IcingaDB>::SetHost(const String& value, bool suppress_events, const Value& cookie)
{
	m_Host = value;
	if (!suppress_events)
		NotifyHost(cookie);
}

void ObjectImpl<IcingaDB>::SetPath(const String& value, bool suppress_events, const Value& cookie)
{
	m_Path = value;
	if (!suppress_events)
		NotifyPath(cookie);
}

void ObjectImpl<IcingaDB>::SetPassword(const String& value, bool suppress_events, const Value& cookie)
{
	m_Password = value;
	if (!suppress_events)
		NotifyPassword(cookie);
}

void ObjectImpl<IcingaDB>::SetPort(int value, bool suppress_events, const Value& cookie)
{
	m_Port = value;
	if (!suppress_events)
		NotifyPort(cookie);
}

void ObjectImpl<IcingaDB>::SetDbIndex(int value, bool suppress_events, const Value& cookie)
{
	m_DbIndex = value;
	if (!suppress_events)
		NotifyDbIndex(cookie);
}

void ObjectImpl<IcingaDB>::NotifyHost(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostChanged(static_cast<IcingaDB *>(this), cookie);
}

void ObjectImpl<IcingaDB>::NotifyPath(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPathChanged(static_cast<IcingaDB *>(this), cookie);
}

void ObjectImpl<IcingaDB>::NotifyPassword(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPasswordChanged(static_cast<IcingaDB *>(this), cookie);
}

void ObjectImpl<IcingaDB>::NotifyPort(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPortChanged(static_cast<IcingaDB *>(this), cookie);
}

void ObjectImpl<IcingaDB>::NotifyDbIndex(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnDbIndexChanged(static_cast<IcingaDB *>(this), cookie);
}

String ObjectImpl<IcingaDB>::GetDefaultHost() const
{
	 return "127.0.0.1"; 
}

String ObjectImpl<IcingaDB>::GetDefaultPath() const
{
	return String();
}

String ObjectImpl<IcingaDB>::GetDefaultPassword() const
{
	return String();
}

int ObjectImpl<IcingaDB>::GetDefaultPort() const
{
	 return 6380; 
}

int ObjectImpl<IcingaDB>::GetDefaultDbIndex() const
{
	return int();
}


boost::signals2::signal<void (const intrusive_ptr<IcingaDB>&, const Value&)> ObjectImpl<IcingaDB>::OnHostChanged;


boost::signals2::signal<void (const intrusive_ptr<IcingaDB>&, const Value&)> ObjectImpl<IcingaDB>::OnPathChanged;


boost::signals2::signal<void (const intrusive_ptr<IcingaDB>&, const Value&)> ObjectImpl<IcingaDB>::OnPasswordChanged;


boost::signals2::signal<void (const intrusive_ptr<IcingaDB>&, const Value&)> ObjectImpl<IcingaDB>::OnPortChanged;


boost::signals2::signal<void (const intrusive_ptr<IcingaDB>&, const Value&)> ObjectImpl<IcingaDB>::OnDbIndexChanged;

void ObjectImpl<IcingaDB>::ValidateDbIndex(const Lazy<int>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateDbIndex(lvalue, utils);
}

void ObjectImpl<IcingaDB>::ValidateHost(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHost(lvalue, utils);
}

void ObjectImpl<IcingaDB>::ValidatePassword(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePassword(lvalue, utils);
}

void ObjectImpl<IcingaDB>::ValidatePath(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePath(lvalue, utils);
}

void ObjectImpl<IcingaDB>::ValidatePort(const Lazy<int>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePort(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
